////////////////////////////////////////////////////////////////////////////
//
//  uart.c
//
//	Einfache UART Implementierung (nur beispielhaft)
//  Enthlt einen Buffer in dem Eingaben interruptgesteuert
//  entgegengenommen werden. 
//
//	Author 	: Rene Bllhoff
//  Created : around Apr 10
//
//	(c) 2005-2010 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
//
//////////////////////////////////////////////////////////////////////////////
//
//  This source file may be used and distributed without
//  restriction provided that this copyright statement is not
//  removed from the file and that any derivative work contains
//  the original copyright notice and the associated disclaimer.
//
//  This program is free software; you can redistribute it and/or
//  modify it under the terms of the GNU General Public License
//  as published by the Free Software Foundation; either version 2
//  of the License, or (at your option) any later version.
//										
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
//  General Public License for more details.
//
//  You should have received a copy of the GNU General Public
//  License along with this source; if not, download it	from
//  http://www.gnu.org/licenses/gpl.html	
//
////////////////////////////////////////////////////////////////////////////	

	#include "lib.h"


  // Eingangsbuffer 
	volatile char 	acUARTRecData [UART_INPUT_LENGTH];
  // Eingangsbuffer-Index 
	volatile U8 		ucUARTRPtr;
  // Lnge der Eingabe (entspricht ucUARTRPtr)
	volatile U8 		ucUARTInputLength;
	// Flag "Eingabe (mit Return abgeschlossen) liegt vor"
	volatile B8   	bUARTProcessInput  	= FALSE;
	// Zuletzt empfangenes Zeichen
	volatile U8  		ucUARTReceivedInput;
	// Flag fr "Zeichen empfangen
	volatile B8   	bUARTInputReceived 	= FALSE;


	// Ausgabe-Funktion und -File fr printf
	extern int uart_putchar (char x, FILE *stream);
	// "Datei" fr printf erstellen
	FILE uartfile 		= FDEV_SETUP_STREAM (uart_putchar, NULL, _FDEV_SETUP_WRITE);

// Initialisierung des UART auf 38400bd bei 16MHz
void vUARTInit (void)
{
	UBRRL = 25;  // 16MHz / (16*38.4kbd) = 26 -> 26-1 = 25 eff. Baudrate 38462bd
	UBRRH = 0;

	UCSRB |= (1 << RXEN)  | (1 << TXEN);			// sender & empfnger einschalten
	UCSRC |= (1 << URSEL) | (3 << UCSZ0); 		// uart mode
  UCSRB |= (1 << RXCIE);										// uart interrupt einschalten

	DDRD  |= 1 << 1;													// tx-pin auf ausgang

	// eingangsbuffer intialisieren
	memset ((void *) acUARTRecData, 0, sizeof (acUARTRecData));

	// variablen initialisieren
	ucUARTRPtr 			 		= 0;
	ucUARTInputLength 	= 0;
	bUARTProcessInput 	= FALSE;
	ucUARTReceivedInput = 0;
	bUARTInputReceived  = FALSE;

	// Ausgabe-"Datei" fr stdout von printf verwenden
	stdout = &uartfile;
}

// einzelnes Zeichen senden
void vUARTSendChar (char cValue)
{
	while (!(UCSRA & (1 << UDRE)));
	UDR = cValue;
}

// Ausgabe-Funktion fr printf
int uart_putchar (char x, FILE *stream)
{
	vUARTSendChar (x);
	return 0;
}

// String aus RAM senden
void vUARTSendString (char *pcString)
{
  while (*pcString) { vUARTSendChar (*pcString++); };
}

// String aus Flash senden
void vUARTSendString_P (char *pcString)
{
  while (pgm_read_byte (pcString)) { vUARTSendChar (pgm_read_byte (pcString++)); };
}

// Empfangsinterrupt
ISR (USART_RXC_vect)
{
	U8 ucData = UDR; 	// Daten holen

	// Emfpangsflag setzen und daten speichern
	bUARTInputReceived 	= TRUE;
	ucUARTReceivedInput = ucData; 

	// Max. Eingabelnge begrenzen
	if (ucUARTRPtr > UART_INPUT_LENGTH) { ucUARTRPtr = UART_INPUT_LENGTH; }
	// Return ?
  if (ucData == 0x0d)
	{
		// Eingabe-String abschlieen und Flag sowie weitere Informationen setzen
	  acUARTRecData [ucUARTRPtr] = 0;
		bUARTProcessInput = TRUE; ucUARTInputLength = ucUARTRPtr; ucUARTRPtr = 0;
	}
	// Delete ? (\x7f daher weil beim vt100 del std-mssig 0x7f ist)
	else if ((ucData == 0x08) || (ucData == 0x7f))
	{ 
		// wenn noch zeichen da, dann einen zurcksetzen
		if (ucUARTRPtr > 0)
		{
			ucUARTRPtr--;
			acUARTRecData [ucUARTRPtr] = 0; 
		}
	}
	// jedes andere zeichen einfach wegspeichern
	else
	{ 
		acUARTRecData [ucUARTRPtr] = ucData; 
		ucUARTRPtr++;
	}
}
